import sqlite3
from sqlite3 import Error
import pandas as pd
 
 
def makeDB(db_file):
    """ create a database connection to a SQLite database """
    global conn
    try:
        conn = sqlite3.connect(db_file)
        #print(sqlite3.version)
    except Error as e:
        print(e)
    finally:
        conn.close()
 

#
#filename="script"
#con=sqlite3.connect(filename+".db")
#wb=pd.ExcelFile(filename+'.xlsx')
#for sheet in wb.sheet_names:
#        df=pd.read_excel(filename+'.xlsx',sheetname=sheet)
#        df.to_sql(sheet,con, index=False,if_exists="replace")
#con.commit()
#con.close()


import xlrd
from pylab import *

def readExcel(file_name, sheet_name):
    ''' it reads excel files. It uses xlrd.
    
    USAGE: from cakemix.excel import readExcel
    data=readExcel('C:/Users/TLB/Documents/file.xlsx','Sheet2')
    
    Last updated: 8/3/2019
    '''
    workbook=xlrd.open_workbook(file_name)

    sheet1=workbook.sheet_by_name(sheet_name)

    data=[]

    for i in range(sheet1.nrows):
        data.append(sheet1.row_values(i))
    
    return data


def get_column(data,col):
    
    ''' it extracts the specific column from a data. It is used with readExcel function.
    USAGE: 
    from cakemix.excel import get_column
    col1=get_column(data2,5)
    
    '''
    result=[]
    for row in data:
        result.append(row[col])
    return result
	
def col2Num(col):
	'''It converts the Excel column name to number
	USAGE:
	num = col2Num("bz")
	'''
	
	dict={"letter":["A","B","C","D","E","F","G","H","I","J","K","L","M","N","O","P","Q","R","S","T","U","V","W","X","Y","Z","AA",'AB','AC','AD','AE','AF','AG','AH','AI','AJ','AK','AL','AM','AN','AO','AP','AQ','AR','AS','AT','AU','AV','AW','AX','AY','AZ','BA','BB','BC','BD','BE','BF','BG','BH','BI','BJ','BK','BL','BM','BN','BO','BP','BQ','BR','BS','BT','BU','BV','BW','BX','BY','BZ'],"num":[0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,52,53,54,55,56,57,58,59,60,61,62,63,64,65,66,67,68,69,70,71,72,73,74,75,76,77,78,79,80]}
	
	num=dict["num"][dict["letter"].index(col.upper())]
	
	return num


def plotData(data):
    ''' placeholder, dont use yet'''
    yield_data=[]

    process=get_column(data,2)[4:20]

    yield_data=get_column(data,6)[4:20]



    plot(yield_data,'ro-')
    ylim(0.7, 1.1)
    show()
	
	
def plotBarData(data1,data2,ylabel,title):
	# cakemix.plotBarData(['Python','C+',"java","Perl","scala","lisp"],[10,8,6,4,5,1],'Usage','programming language')
	import numpy as np
	import matplotlib .pyplot as plt

	#data1=['Python','C+',"java","Perl","scala","lisp"]

	#data2=[10,8,6,4,2,1]

	plt.bar(data1,data2,alpha=0.5)
	#plt.ylabel('Usage')
	plt.ylabel(ylabel)
	#plt.title('programming language usage')
	plt.title(title)
	plt.show()

def plotLineData(data1,xlabel,ylabel,title):
	# cakemix.plotLineData([1, 2, 4, 8, 16, 32, 64, 128, 256],'model','xlabel','variance')
	from matplotlib import pyplot as plt
	data = [i for i, _ in enumerate(data1)]
	plt.plot(data, data1,     'g-',  label='variance') 
	plt.xlabel(xlabel)
	plt.ylabel(ylabel) 	
	plt.title(title) 
	plt.show()

def plotScatterData(data1,data2,labels,xlabel,ylabel,title):
	# cakemix.plotScatterData([70,65,72, 63, 71, 64, 60, 64, 67],[175,170,205,120,220,130,105,145,190],['a','b','c','d','e','f','g','h','i'],'# of Friends','daily minutes spent on the site','Daily Minutes vs Number of Friends')
	from matplotlib import pyplot as plt

	friends = [70,65,72, 63, 71, 64, 60, 64, 67]
	minutes = [175,170,205,120,220,130,105,145,190]

	labels = ['a','b','c','d','e','f','g','h','i']

	plt.scatter(data1,data2)
	for label, data1_count, data2_count in zip(labels, data1, data2):
		plt.annotate(label,xy=(data1_count,data2_count),xytext = (5,-5),textcoords='offset points')

	plt.title('Daily Minutes vs Number of Friends')
	plt.xlabel('# of Friends')
	plt.ylabel('daily minutes spent on the site')
	
	plt.title(title)
	plt.xlabel(xlabel)
	plt.ylabel(ylabel)
	plt.show()
	
def plotHistData(data,label,title):
    ''' Placelholder. not yet useful. need to modify '''
    import matplotlib.pyplot as plt
    plt.hist(data,10, histtype='bar', align='mid',color='c',label=label,edgecolor='black')
    plt.legend()
    plt.title(title)
    plt.show()

	


import re
from collections import Counter

def words(text): return re.findall(r'\w+', text.lower())

WORDS = Counter(words(open('10000-english-words.txt').read()))

def P(word, N=sum(WORDS.values())): 
    "Probability of `word`."
    return WORDS[word] / N

def correction(word): 
    "Most probable spelling correction for word."
    return max(candidates(word), key=P)

def candidates(word): 
    "Generate possible spelling corrections for word."
    return (known([word]) or known(edits1(word)) or known(edits2(word)) or [word])

def known(words): 
    "The subset of `words` that appear in the dictionary of WORDS."
    return set(w for w in words if w in WORDS)

def edits1(word):
    "All edits that are one edit away from `word`."
    letters    = 'abcdefghijklmnopqrstuvwxyz'
    splits     = [(word[:i], word[i:])    for i in range(len(word) + 1)]
    deletes    = [L + R[1:]               for L, R in splits if R]
    transposes = [L + R[1] + R[0] + R[2:] for L, R in splits if len(R)>1]
    replaces   = [L + c + R[1:]           for L, R in splits if R for c in letters]
    inserts    = [L + c + R               for L, R in splits for c in letters]
    return set(deletes + transposes + replaces + inserts)

def edits2(word): 
    "All edits that are two edits away from `word`."
    return (e2 for e1 in edits1(word) for e2 in edits1(e1))


# -*- coding: utf-8 -*-

import pyttsx3



def speak(text):
    '''Example: for single recipient: 
            speak(""I am a robot who can speak the text you write.")
    '''
    engine = pyttsx3.init()
    engine.say(text)
    engine.runAndWait()



    




# -*- coding: utf-8 -*-
'''
#import win32com.client as win32
import datetime

now = datetime.datetime.now()

#def sendEmail(mailTo,subject,body):
    Example: for single recipient: 
            sendEmail('email@example.com','First email subject','Email body as how are you?') 
        for multiple recipients: 
            sendEmail('email@example.com;email1@example.com','First email subject','Email body as how are you?') 
    You need to have Outlook installed on your computer and set for sending emails
    '''
 '''   outlook = win32.Dispatch('outlook.application')
    mail = outlook.CreateItem(0)
    mail.To = mailTo
    mail.Subject = subject
    mail.Body = body
    #mail.HTMLBody = '''<html>'''

    #To attach a file to the email (optional):
    #attachment  = "e:\\data\python\\plot3.png"
    #mail.Attachments.Add(attachment)
    
    mail.Send()
    
    '''




def findFilesDirectory(root,pattern):
    '''
    inspired by the following entry:
    https://stackoverflow.com/questions/2909975/python-list-directory-subdirectory-and-files
    
    USAGE: data=findFilesDirectory('e:\data','*.csv')
    '''
    import os
    from fnmatch import fnmatch

    #root = 'c:/data'
    #pattern = "*.xlsx"
    out1=[]
    out2=[]
    for path, subdirs, files in os.walk(root):
        for name in files:
            if fnmatch(name, pattern):
                out1=os.path.join(path, name)
                out2.append(out1)
                #print(os.path.join(path, name))
    return out2


def findUniqueList(param1):
    ''' it returns the unique subset of the list
	USAGE: findUniqueList([1,3,3,4,5,5,6])
	'''
    used=set()
    
    
    if len(param1) < 2:
        print("this is not a list")
    else:
        unique_list = [x for x in param1 if x not in used and (used.add(x) or True)]
        return unique_list
    

def extractData(data, textSearched):
    ''' 
    USAGE: extractData(['orange','apple','apple1','banana'],'apple') 
    '''
    
    index_page=[]
    
    for i in range(0,len(data),1):
        if textSearched in data[i]:
            index_page.append(data[i].strip())
    return index_page



def isMatch(name1,name2):
    ''' it returns if two names are related. 
    USAGE: isMatch('Apple juice','apple Juice')
    if result is 0-> two names are identical
    if result is more than 0 but less than 15deg, the names have similarity
    if the result is more than 30deg, the names have no similarity
    '''
    import math
    from collections import Counter

    def build_vector(iterable1, iterable2):
        counter1 = Counter(iterable1)
        counter2 = Counter(iterable2)
        all_items = set(counter1.keys()).union(set(counter2.keys()))
        vector1 = [counter1[k] for k in all_items]
        vector2 = [counter2[k] for k in all_items]
        return vector1, vector2

    def cosim(v1, v2):
        dot_product = sum(n1 * n2 for n1, n2 in zip(v1, v2) )
        magnitude1 = math.sqrt(sum(n ** 2 for n in v1))
        magnitude2 = math.sqrt(sum(n ** 2 for n in v2))
        return dot_product / (magnitude1 * magnitude2)
    
    l1 = name1.lower().split()
    l2 = name2.lower().split()

#l1 = "A cover".lower().split()
#l2 = "a cover".lower().split()


    v1, v2 = build_vector(l1, l2)
    result=round(math.degrees(math.acos(cosim(v1, v2))),5)
    
    return result


# -*- coding: utf-8 -*-
"""
Created on Wed Oct  2 14:57:06 2019

@author: Z2 Mini
"""

from tkinter import ttk
from tkinter import *

import sqlite3

class Product:
    # connection dir property
    db_name = 'database.db'

    def __init__(self, window):
        # Initializations 
        self.wind = window
        self.wind.title('Products Application')

        # Creating a Frame Container 
        frame = LabelFrame(self.wind, text = 'Register new Product')
        frame.grid(row = 0, column = 0, columnspan = 3, pady = 20)

        # Name Input
        Label(frame, text = 'Name: ').grid(row = 1, column = 0)
        self.name = Entry(frame)
        self.name.focus()
        self.name.grid(row = 1, column = 1)

        # Price Input
        Label(frame, text = 'Price: ').grid(row = 2, column = 0)
        self.price = Entry(frame)
        self.price.grid(row = 2, column = 1)

        # Button Add Product 
        ttk.Button(frame, text = 'Save Product', command = self.add_product).grid(row = 3, columnspan = 2, sticky = W + E)

        # Output Messages 
        self.message = Label(text = '', fg = 'red')
        self.message.grid(row = 3, column = 0, columnspan = 2, sticky = W + E)

        # Table
        self.tree = ttk.Treeview(height = 10, columns = 2)
        self.tree.grid(row = 4, column = 0, columnspan = 2)
        self.tree.heading('#0', text = 'Name', anchor = CENTER)
        self.tree.heading('#1', text = 'Price', anchor = CENTER)

        # Buttons
        ttk.Button(text = 'DELETE', command = self.delete_product).grid(row = 5, column = 0, sticky = W + E)
        ttk.Button(text = 'EDIT', command = self.edit_product).grid(row = 5, column = 1, sticky = W + E)

        # Filling the Rows
        self.get_products()

    # Function to Execute Database Querys
    def run_query(self, query, parameters = ()):
        with sqlite3.connect(self.db_name) as conn:
            cursor = conn.cursor()
            result = cursor.execute(query, parameters)
            conn.commit()
        return result

    # Get Products from Database
    def get_products(self):
        # cleaning Table 
        records = self.tree.get_children()
        for element in records:
            self.tree.delete(element)
        # getting data
        query = 'SELECT * FROM product ORDER BY name DESC'
        db_rows = self.run_query(query)
        # filling data
        for row in db_rows:
            self.tree.insert('', 0, text = row[1], values = row[2])

    # User Input Validation
    def validation(self):
        return len(self.name.get()) != 0 and len(self.price.get()) != 0

    def add_product(self):
        if self.validation():
            query = 'INSERT INTO product VALUES(NULL, ?, ?)'
            parameters =  (self.name.get(), self.price.get())
            self.run_query(query, parameters)
            self.message['text'] = 'Product {} added Successfully'.format(self.name.get())
            self.name.delete(0, END)
            self.price.delete(0, END)
        else:
            self.message['text'] = 'Name and Price is Required'
        self.get_products()

    def delete_product(self):
        self.message['text'] = ''
        try:
           self.tree.item(self.tree.selection())['text'][0]
        except IndexError as e:
            self.message['text'] = 'Please select a Record'
            return
        self.message['text'] = ''
        name = self.tree.item(self.tree.selection())['text']
        query = 'DELETE FROM product WHERE name = ?'
        self.run_query(query, (name, ))
        self.message['text'] = 'Record {} deleted Successfully'.format(name)
        self.get_products()

    def edit_product(self):
        self.message['text'] = ''
        try:
            self.tree.item(self.tree.selection())['values'][0]
        except IndexError as e:
            self.message['text'] = 'Please, select Record'
            return
        name = self.tree.item(self.tree.selection())['text']
        old_price = self.tree.item(self.tree.selection())['values'][0]
        self.edit_wind = Toplevel()
        self.edit_wind.title = 'Edit Product'
        # Old Name
        Label(self.edit_wind, text = 'Old Name:').grid(row = 0, column = 1)
        Entry(self.edit_wind, textvariable = StringVar(self.edit_wind, value = name), state = 'readonly').grid(row = 0, column = 2)
        # New Name
        Label(self.edit_wind, text = 'New Price:').grid(row = 1, column = 1)
        new_name = Entry(self.edit_wind)
        new_name.grid(row = 1, column = 2)

        # Old Price 
        Label(self.edit_wind, text = 'Old Price:').grid(row = 2, column = 1)
        Entry(self.edit_wind, textvariable = StringVar(self.edit_wind, value = old_price), state = 'readonly').grid(row = 2, column = 2)
        # New Price
        Label(self.edit_wind, text = 'New Name:').grid(row = 3, column = 1)
        new_price= Entry(self.edit_wind)
        new_price.grid(row = 3, column = 2)

        Button(self.edit_wind, text = 'Update', command = lambda: self.edit_records(new_name.get(), name, new_price.get(), old_price)).grid(row = 4, column = 2, sticky = W)
        self.edit_wind.mainloop()

    def edit_records(self, new_name, name, new_price, old_price):
        query = 'UPDATE product SET name = ?, price = ? WHERE name = ? AND price = ?'
        parameters = (new_name, new_price,name, old_price)
        self.run_query(query, parameters)
        self.edit_wind.destroy()
        self.message['text'] = 'Record {} updated successfylly'.format(name)
        self.get_products()

if __name__ == '__main__':
    window = Tk()
    application = Product(window)
    window.mainloop()


from bs4 import BeautifulSoup
import urllib
import re

html_page = urllib.urlopen("https://arstechnica.com")
soup = BeautifulSoup(html_page)
for link1 in soup.findAll('a', attrs={'href': re.compile("^http://")}):
    print(link1.get('href'))




from bs4 import BeautifulSoup
from urllib.request import urlopen
import re


def openURL(url):
	'''
	It opens the website link and extract the page content.
	'''

	f = Request.urlopen(url)
	# Read from the object, storing the page's contents in 's'.
	s = f.readlines()
	f.close()
	return s


def getLinks(url):
	#It looks for links from the link and populates those links.
    html_page = urlopen(url)
    soup = BeautifulSoup(html_page)
    links = []

    for link in soup.findAll('a', attrs={'href': re.compile("^http://")}):
        links.append(link.get('href'))
 
    return links






import setuptools
import os

with open(os.path.join(os.getcwd(), "README.md"), "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="cakemix",
    version="0.0.12",
    author="Mesut Varlioglu, PhD",
    author_email="varlioglu@gmail.com",
    description="Python library to open Office documents, automate the data analysis and making presentations",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/varlmes/cakemix",
    packages=setuptools.find_packages(),
    install_requires=[
        "bs4",
        "matplotlib",
        "numpy",
        "xlrd",
		"flask",
        "pyttsx3"
    ],
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    zip_safe=False
)



import requests
import threading
import time
import webbrowser


from flask import Flask
app = Flask(__name__)

#@app.before_first_request
#def activate_job():
#    def run_job():
#        while True:
#            print("Run recurring task")
#            time.sleep(3)
#
#    thread = threading.Thread(target=run_job)
#    thread.start()

#def start_flask():
#    webbrowser.open_new('http://127.0.0.1:5000/')  # Go to flask website
#    #start_runner()
#    if __name__ == "__main__":
#        
#        app.run()

@app.route("/")
def hello():
    return "Hello World!"


def start_runner():
    def start_loop():
        not_started = True
        while not_started:
            print('In start loop')
            try:
                r = requests.get('http://127.0.0.1:5000/')
                if r.status_code == 200:
                    print('Server started, quiting start_loop')
                    not_started = False
                print(r.status_code)
            except:
                print('Server not yet started')
            time.sleep(2)

    print('Started runner')
    thread = threading.Thread(target=start_loop)
    thread.start()

if __name__ == "__main__":
    webbrowser.open_new('http://127.0.0.1:5000/')  # Go to flask website
    #start_runner()
    app.run()



# -*- coding: utf-8 -*-
"""
cakemix
~~~~~

Cakemix is a Python library to make the business life easier. It uses standard Python libraries and simplifies opening/writing excel spreadsheets, 
comparing data in several spreadsheets, making ppt files, to reading pdf files.
It also has functions to automate the routine work like emailing, sending
reminders.

:copyright: © 2022 by Mesut Varlioglu, PhD.
:license: BSD, see LICENSE.rst for more details.
"""

#List classes
from .list import findUniqueList, findFilesDirectory, extractData, isMatch

#Web classes
from .web import openURL, getLinks

#Excel classes
from .excel import readExcel, get_column, plotBarData, plotHistData, plotLineData, plotScatterData

#Flask GUI classes
from .flask_gui import *

#Email classes
#from .e_mail import sendEmail

#Database classes
from .db import makeDB

#Spell classes
#from .spell import correction, candidates, known, edits1, edits2

#Text-To-Speech Classes
from .text_speak import speak


name = "cakemix"
__version__ = '0.0.12'
